<?php
require_once(dirname(__FILE__) . "/../../../classes/PluginsClassiques.class.php");
require_once(dirname(__FILE__) . "/../../../classes/Tlog.class.php");
require_once(dirname(__FILE__) . "/../../../classes/Variable.class.php");
require_once(dirname(__FILE__) . "/lessphp/lessc.inc.php");
require_once(dirname(__FILE__) . "/ExtensionFilterIteratorDecorator.class.php");

class Tless extends PluginsClassiques {
    
    private $_directories = array();

    public function __construct($nom = "tless") {
        $this->addDirectory(dirname(__FILE__) . "/../../../template/");
        parent::__construct($nom);
        
    }
    
    /**
     * 
     * put in $_directories new directory to parse
     * 
     * @param string $directory
     */
    public function addDirectory($directory){
        if(false === array_search($directory, $this->_directories)){
            array_push($this->_directories, $directory);
        }
    }
    
    
    /**
     * 
     * Replace all the values in $_directories
     * 
     * @param array $directories
     */
    public function setDirectories(array $directories){
        $this->_directories = $directories;
    }
    
    
    public function init() {
        Variable::ecrire('tless_dev', '1', true, 1, 1);
        Variable::ecrire('tless_file', 'styles_less.css', true, 1, 1);
    }
    
    public function clear_cache(){
        $this->compileLess((Variable::lire('tless_dev'))?true:false);
    }
    
    
    /**
     * If shop is in dev mode, all time we check if less files can be compile
     * 
     */
    public function post(){        
        if(Variable::lire('tless_dev')){
            $this->compileLess(true);
        }
    }
    
    /**
     * 
     * Compile all the less files find in directories defined in  $_directories
     * 
     * @param bool $dev
     */
    public function compileLess($dev = false){
        
        $input = $this->findLess();
        
        if(false === is_null($input)){
            $less = new lessc();
            
            if(false === $dev){
                $less->setFormatter("compressed");
            }
            try {
                $output = $less->compile($input);
                file_put_contents(dirname(__FILE__)."/../../../template/".Variable::lire('tless_file','styles_less.css'), $output);
            }
            catch(Exception $e){
                if(false === $dev){
                    Tlog::error("error during compiling lesscss with message : ".$e->getMessage());
                }
                else{
                    die($e->getMessage());
                }
            }
        }
    }
    
    /**
     * @see compileLess
     */
    public static function compile(){
        $tless = new self;
        $tless->compileLess();
    }
    
    /**
     * 
     * Find all the less files and return the concatenation of all this files
     * 
     * @return string
     */
    public function findLess(){
        $input = null;
        foreach($this->_directories as $directory){
            $directories = new AppendIterator ();
            
            $directories->append(new RecursiveIteratorIterator (new RecursiveDirectoryIterator ($directory)));
            
            $files = new ExtensionFilterIteratorDecorator($directories);
            $files->setExtension(".less");
            
            foreach($files as $file){
                $input .= file_get_contents($file);
            }
        }
        
        return $input;
    }
}

?>
